@@ -3,9 +3,11 @@ use strict;
 use warnings;
 use v5.20;
 use experimental qw/ signatures /;
+
 package YAML::Tidy::Node;
 
-our $VERSION = '0.002_002'; # TRIAL VERSION
+our $VERSION = '0.003_001'; # TRIAL VERSION
+use overload '""' => \&_stringify;
 
 sub new($class, %args) {
     my $self = {
@@ -14,26 +16,31 @@ sub new($class, %args) {
     return bless $self, $class;
 }
 
-sub pre($self, $node) {
-    my $index = $node->{index} - 1;
-    my $end;
-    if ($index < 1) {
-        $end = $self->open->{end};
+sub _stringify($self, @args) {
+    my $type = $self->{type} // '';
+    my $str = "($type)";
+    if ($self->is_collection) {
+        my $open = $self->open;
+        my $close = $self->close;
+        $str .= sprintf " <L %d C %d> <L %d C %d> - <L %d C %d> <L %d C %d>",
+            $open->{start}->{line}, $open->{start}->{column},
+            $open->{end}->{line}, $open->{end}->{column},
+            $close->{start}->{line}, $close->{start}->{column},
+            $close->{end}->{line}, $close->{end}->{column},
     }
     else {
-        my $previous = $self->{children}->[ $index -1 ];
-        $end = $previous->end;
+        $str .= sprintf " <L %d C %d> - <L %d C %d>",
+            $self->start->{line}, $self->start->{column},
+            $self->end->{line}, $self->end->{column},
     }
-    return $end;
+    return $str;
 }
 
-
 package YAML::Tidy::Node::Collection;
+use constant DEBUG => $ENV{YAML_TIDY_DEBUG} ? 1 : 0;
 
 use base 'YAML::Tidy::Node';
 
-#sub is_scalar { 0 }
-
 sub is_collection { 1 }
 
 sub indent($self) {
@@ -41,7 +48,6 @@ sub indent($self) {
     if ($firstevent->{name} eq 'document_start_event') {
         return 0;
     }
-
     my $startcol = $firstevent->{end}->{column};
     return $startcol;
 }
@@ -53,10 +59,6 @@ sub end($self) {
     return $self->close->{end};
 }
 
-sub closestart($self) {
-    return $self->close->{start};
-}
-
 sub realendline($self) {
     $self->close->{end}->{line} - 1;
 }
@@ -102,7 +104,29 @@ sub _fix_flow_indent($self, %args) {
     }
 }
 
+sub fix_lines($self, $startline, $diff) {
+    my $start = $self->open;
+    DEBUG and warn __PACKAGE__.':'.__LINE__.": ======== fix_lines $startline $diff ($self)\n";
+    my $end = $self->close;
+    for my $pos ($start->{start}, $start->{end}) {
+        if ($pos->{line} >= $startline) {
+            $pos->{line} += $diff;
+        }
+    }
+    for my $pos ($end->{start}, $end->{end}) {
+        if ($pos->{column} == 0 and $pos->{line} == $startline) {
+        }
+        elsif ($pos->{line} >= $startline) {
+            $pos->{line} += $diff;
+        }
+    }
+    for my $c (@{ $self->{children} }) {
+        $c->fix_lines($startline, $diff);
+    }
+}
+
 package YAML::Tidy::Node::Scalar;
+use constant DEBUG => $ENV{YAML_TIDY_DEBUG} ? 1 : 0;
 use YAML::PP::Common qw/
     YAML_PLAIN_SCALAR_STYLE YAML_SINGLE_QUOTED_SCALAR_STYLE
     YAML_DOUBLE_QUOTED_SCALAR_STYLE YAML_LITERAL_SCALAR_STYLE
@@ -112,8 +136,6 @@ use YAML::PP::Common qw/
 
 use base 'YAML::Tidy::Node';
 
-#sub is_scalar { 1 }
-
 sub is_collection { 0 }
 
 sub indent($self) {
@@ -131,18 +153,14 @@ sub end($self) {
     return $self->close;
 }
 
-sub closestart($self) {
-    return $self->close;
-}
-
 sub realendline($self) {
     my $end = $self->close;
     if ($self->{style} == YAML_LITERAL_SCALAR_STYLE
         or $self->{style} == YAML_FOLDED_SCALAR_STYLE) {
-            if ($end->{column} == 0) {
-                return $end->{line} - 1;
-            }
+        if ($end->{column} == 0) {
+            return $end->{line} - 1;
         }
+    }
     $end->{line};
 }
 
@@ -170,7 +188,6 @@ sub empty_scalar($self) {
     return 0;
 }
 
-
 sub fix_node_indent($self, $fix) {
     for my $pos ($self->open, $self->close) {
         $pos->{column} += $fix;
@@ -187,4 +204,22 @@ sub _fix_flow_indent($self, %args) {
     }
 }
 
+sub fix_lines($self, $startline, $diff) {
+    DEBUG and warn __PACKAGE__.':'.__LINE__.": ======== fix_lines $startline $diff ($self)\n";
+    for my $pos ($self->open) {
+        if ($self->empty_scalar and $pos->{column} == 0 and $pos->{line} == $startline) {
+        }
+        elsif ($pos->{line} >= $startline) {
+            $pos->{line} += $diff;
+        }
+    }
+    for my $pos ($self->close) {
+        if ($pos->{column} == 0 and $pos->{line} == $startline) {
+        }
+        elsif ($pos->{line} >= $startline) {
+            $pos->{line} += $diff;
+        }
+    }
+}
+
 1;
